 MODULE mod_chem_spack_kinetic
  
   IMPLICIT NONE
   PRIVATE
   PUBLIC :: kinetic ! subroutine
 CONTAINS
   SUBROUTINE kinetic(jppj,Jphoto,rk,temp,xlw,Press,cosz,att,ijkbeg,ijkend,maxblock_size,nr)
 
!------------------------------------------------------------------------
!
!     -- DESCRIPTION
!
!     This routine computes the kinetic rates for the gas-phase.
!     This routine is automatically generated by SPACK.
!     Mechanism: ../Mechanism/RELACS 
!     Species: ../Mechanism/ciRLCS 
!
!------------------------------------------------------------------------
!
!     -- INPUT VARIABLES
!
!     TEMP: temperature ([K]).
!     XLW: water massic fraction.
!     PRESS: pressure ([Pa]).
!     ATT: attenuation variable.
!
!     -- INPUT/OUTPUT VARIABLES
!
!     -- OUTPUT VARIABLES
!
!     RK: kinetic rates.
!
!------------------------------------------------------------------------
!
!     -- REMARKS
!
!------------------------------------------------------------------------
!
!     -- MODIFICATIONS
!
!------------------------------------------------------------------------
!
!     -- AUTHOR(S)
!
!     SPACK.
!
!------------------------------------------------------------------------
 
      IMPLICIT NONE
 
 
 
      INTEGER,INTENT(IN) :: jppj,ijkbeg,ijkend,maxblock_size,nr
      DOUBLE PRECISION,INTENT(IN) ::  xlw(maxblock_size),att(maxblock_size),cosz(maxblock_size)
      DOUBLE PRECISION,INTENT(IN) ::  temp(maxblock_size),Press(maxblock_size)
      DOUBLE PRECISION,INTENT(OUT) ::  rk(maxblock_size,nr)
      DOUBLE PRECISION,DIMENSION(maxblock_size) ::  Effko,Rapk,facteur
      DOUBLE PRECISION,DIMENSION(maxblock_size) :: YlH2O,SumM,azi
      DOUBLE PRECISION,INTENT(IN) ::  Jphoto(maxblock_size,jppj)
      INTEGER :: ijk
 
!     Compute third body.
!     Conversion = Avogadro*1d-6/Perfect gas constant.
!     PRESS in Pascal, SUMM in molecules/cm3, TEMP in Kelvin
 
      DO ijk=ijkbeg,ijkend
        SumM(ijk) = Press(ijk) * 7.243D16 / temp(ijk)
      END DO
 
!     Number of water molecules computed from the massic fraction
!     (absolute humidity)
 
      DO ijk=ijkbeg,ijkend
         YlH2O(ijk) = 29.d0*SumM(ijk)*xlw(ijk)/(18.d0+11.d0*xlw(ijk))
      END DO
 
!     For the zenithal angle at tropics
 
 
 
       DO ijk=ijkbeg,ijkend
       rk(ijk,  1) = Jphoto(ijk,  1)
       rk(ijk,  2) = Jphoto(ijk,  2)
       rk(ijk,  3) = Jphoto(ijk,  3)
       rk(ijk,  4) = Jphoto(ijk,  4)
       rk(ijk,  5) = Jphoto(ijk,  5)
       rk(ijk,  6) = Jphoto(ijk,  6)
       rk(ijk,  7) = Jphoto(ijk,  7)
       rk(ijk,  8) = Jphoto(ijk,  8)
       rk(ijk,  9) = Jphoto(ijk,  9)
       rk(ijk, 10) = Jphoto(ijk, 10)
       rk(ijk, 11) = Jphoto(ijk, 11)
       rk(ijk, 12) = Jphoto(ijk, 12)
       rk(ijk, 13) = Jphoto(ijk, 13)
       rk(ijk, 14) = Jphoto(ijk, 14)
       rk(ijk, 15) = Jphoto(ijk, 15)
       rk(ijk, 16) = Jphoto(ijk, 16)
       rk(ijk, 17) = Jphoto(ijk, 17)
      rk(ijk, 18) = SumM(ijk) * 6.0d-34 * (temp(ijk)/3.d2) ** (-2.3d0)
      rk(ijk, 18) = rk(ijk, 18) * SumM(ijk) * 0.2d0
      rk(ijk, 19) =  DEXP(-0.2555157957424871D+02   &
         - (  0.2060000000000000D+04 )/temp(ijk))
      rk(ijk, 20) =  DEXP(-0.2474064935803238D+02   &
         - ( -0.1100000000000000D+03 )/temp(ijk))
      rk(ijk, 20) = rk(ijk, 20) * SumM(ijk) * 0.8d0
      rk(ijk, 21) =  DEXP(-0.2416528521312882D+02   &
         - ( -0.7000000000000000D+02 )/temp(ijk))
      rk(ijk, 21) = rk(ijk, 21) * SumM(ijk) * 0.2d0
      rk(ijk, 22) =  0.2200000000000000D-09
      rk(ijk, 22) = rk(ijk, 22) * YlH2O(ijk)
      rk(ijk, 23) =  DEXP(-0.2716101748668281D+02   &
         - (  0.9400000000000000D+03 )/temp(ijk))
      rk(ijk, 24) =  DEXP(-0.3214088112211231D+02   &
         - (  0.5000000000000000D+03 )/temp(ijk))
      rk(ijk, 25) =  DEXP(-0.2375982010502066D+02   &
         - ( -0.2500000000000000D+03 )/temp(ijk))
      rk(ijk, 26) =  DEXP(-0.2656631037893612D+02   &
         - (  0.1600000000000000D+03 )/temp(ijk))
      rk(ijk, 27) = 2.2d-13 * dexp(620.0d0 / temp(ijk))   &
                    + 1.9d-33* SumM(ijk) * dexp(980.0d0 / temp(ijk))
      rk(ijk, 28) = 3.08d-34 * dexp(2820.0d0 / temp(ijk)) +    &
                    2.66d-54 * SumM(ijk) * dexp(3180.0d0 / temp(ijk))
      rk(ijk, 28) = rk(ijk, 28) * YlH2O(ijk)
      Effko(ijk) =  0.9000000000000000D-31* (temp(ijk) / 3.d2)   &
                   **(- ( 0.1500000000000000D+01))
      Rapk(ijk) =  0.3000000000000000D-10* (temp(ijk) / 3.d2)   &
                    **(- ( 0.0000000000000000D+00))
      rk(ijk, 29) = (Effko(ijk) * SumM(ijk) /    &
                    ( 1.0d0 + Effko(ijk) * SumM(ijk) / Rapk(ijk))) *   &
                    0.6000D+00** (1.0d0 / (1.0d0 +    &
                     (LOG10(Effko(ijk) * SumM(ijk) / Rapk(ijk)))**2))
      rk(ijk, 30) =  DEXP(-0.2575921893902696D+02   &
         - ( -0.1200000000000000D+03 )/temp(ijk))
      Effko(ijk) =  0.9000000000000000D-31* (temp(ijk) / 3.d2)   &
                   **(- ( 0.2000000000000000D+01))
      Rapk(ijk) =  0.2200000000000000D-10* (temp(ijk) / 3.d2)   &
                    **(- ( 0.0000000000000000D+00))
      rk(ijk, 31) = (Effko(ijk) * SumM(ijk) /    &
                    ( 1.0d0 + Effko(ijk) * SumM(ijk) / Rapk(ijk))) *   &
                    0.6000D+00** (1.0d0 / (1.0d0 +    &
                     (LOG10(Effko(ijk) * SumM(ijk) / Rapk(ijk)))**2))
      Effko(ijk) =  0.7000000000000000D-30* (temp(ijk) / 3.d2)   &
                   **(- ( 0.2600000000000000D+01))
      Rapk(ijk) =  0.1500000000000000D-10* (temp(ijk) / 3.d2)   &
                    **(- ( 0.5000000000000000D+00))
      rk(ijk, 32) = (Effko(ijk) * SumM(ijk) /    &
                    ( 1.0d0 + Effko(ijk) * SumM(ijk) / Rapk(ijk))) *   &
                    0.6000D+00** (1.0d0 / (1.0d0 +    &
                     (LOG10(Effko(ijk) * SumM(ijk) / Rapk(ijk)))**2))
      Effko(ijk) =  0.2600000000000000D-29* (temp(ijk) / 3.d2)   &
                   **(- ( 0.3200000000000000D+01))
      Rapk(ijk) =  0.2400000000000000D-10* (temp(ijk) / 3.d2)   &
                    **(- ( 0.1300000000000000D+01))
      rk(ijk, 33) = (Effko(ijk) * SumM(ijk) /    &
                    ( 1.0d0 + Effko(ijk) * SumM(ijk) / Rapk(ijk))) *   &
                    0.6000D+00** (1.0d0 / (1.0d0 +    &
                     (LOG10(Effko(ijk) * SumM(ijk) / Rapk(ijk)))**2))
      rk(ijk, 34) =  0.2200000000000000D-10
      rk(ijk, 35) =  DEXP(-0.2632268829627837D+02   &
         - ( -0.2500000000000000D+03 )/temp(ijk))
      Effko(ijk) =  0.1800000000000000D-30* (temp(ijk) / 3.d2)   &
                   **(- ( 0.3200000000000000D+01))
      Rapk(ijk) =  0.4700000000000000D-11* (temp(ijk) / 3.d2)   &
                    **(- ( 0.1400000000000000D+01))
      rk(ijk, 36) = (Effko(ijk) * SumM(ijk) /    &
                    ( 1.0d0 + Effko(ijk) * SumM(ijk) / Rapk(ijk))) *   &
                    0.6000D+00** (1.0d0 / (1.0d0 +    &
                     (LOG10(Effko(ijk) * SumM(ijk) / Rapk(ijk)))**2))
      rk(ijk, 37) =  DEXP( 0.6142746008608852D+02   &
         - (  0.1090000000000000D+05 )/temp(ijk))
      Effko(ijk) =  0.1800000000000000D-30* (temp(ijk) / 3.d2)   &
                   **(- ( 0.3200000000000000D+01))
      Rapk(ijk) =  0.4700000000000000D-11* (temp(ijk) / 3.d2)   &
                    **(- ( 0.1400000000000000D+01))
      facteur(ijk) = (Effko(ijk) * SumM(ijk) / ( 1.0d0 + Effko(ijk) * SumM(ijk) /    &
                 Rapk(ijk))) * 0.6d0 ** (1.0d0 / (1.0d0 +    &
                  (LOG10(Effko(ijk) * SumM(ijk) / Rapk(ijk)))**2))
      rk(ijk, 37) = facteur(ijk) * rk(ijk, 37)
      rk(ijk, 38) =  0.3500000000000000D-11
      rk(ijk, 39) =  DEXP(-0.2474064935803238D+02   &
         - (  0.3900000000000000D+03 )/temp(ijk))
      Effko(ijk) = 7.2d-15 * dexp(785.0d0 / temp(ijk))
      Rapk(ijk) = 4.1d-16 * dexp(1440.0d0 / temp(ijk))
      facteur(ijk) =1.9d-33 * dexp(725.0d0 / temp(ijk)) * SumM(ijk)
      rk(ijk, 40) = Effko(ijk) + facteur(ijk)/(1.0d0 + facteur(ijk) / Rapk(ijk))
      rk(ijk, 41) =  DEXP(-0.2736865685146106D+02   &
         - ( -0.3800000000000000D+03 )/temp(ijk))
      rk(ijk, 42) =  DEXP(-0.2693787393536860D+02   &
         - (  0.1400000000000000D+04 )/temp(ijk))
      rk(ijk, 43) =  DEXP(-0.2975128465212864D+02   &
         - (  0.2450000000000000D+04 )/temp(ijk))
      rk(ijk, 44) =  0.3300000000000000D-38
      rk(ijk, 44) = rk(ijk, 44) * SumM(ijk) * 0.2d0
      rk(ijk, 45) =  DEXP(-0.2492297091482634D+02   &
         - ( -0.1700000000000000D+03 )/temp(ijk))
      rk(ijk, 46) =  DEXP(-0.3073211390514037D+02   &
         - (  0.1260000000000000D+04 )/temp(ijk))
      Effko(ijk) =  0.2200000000000000D-29* (temp(ijk) / 3.d2)   &
                   **(- ( 0.3900000000000000D+01))
      Rapk(ijk) =  0.1500000000000000D-11* (temp(ijk) / 3.d2)   &
                    **(- ( 0.7000000000000000D+00))
      rk(ijk, 47) = (Effko(ijk) * SumM(ijk) /    &
                    ( 1.0d0 + Effko(ijk) * SumM(ijk) / Rapk(ijk))) *   &
                    0.6000D+00** (1.0d0 / (1.0d0 +    &
                     (LOG10(Effko(ijk) * SumM(ijk) / Rapk(ijk)))**2))
      rk(ijk, 48) =  DEXP( 0.6117554523749536D+02   &
         - (  0.1100000000000000D+05 )/temp(ijk))
      Effko(ijk) =  0.2200000000000000D-29* (temp(ijk) / 3.d2)   &
                   **(- ( 0.3900000000000000D+01))
      Rapk(ijk) =  0.1500000000000000D-11* (temp(ijk) / 3.d2)   &
                    **(- ( 0.7000000000000000D+00))
      facteur(ijk) = (Effko(ijk) * SumM(ijk) / ( 1.0d0 + Effko(ijk) * SumM(ijk) /    &
                 Rapk(ijk))) * 0.6d0 ** (1.0d0 / (1.0d0 +    &
                  (LOG10(Effko(ijk) * SumM(ijk) / Rapk(ijk)))**2))
      rk(ijk, 48) = facteur(ijk) * rk(ijk, 48)
      rk(ijk, 49) =  DEXP(-0.2779354004542632D+02   &
         - (  0.2450000000000000D+04 )/temp(ijk))
      rk(ijk, 50) =  DEXP(-0.2592627302369012D+02   &
         - (  0.2000000000000000D+04 )/temp(ijk))
      rk(ijk, 50) = rk(ijk, 50) * SumM(ijk) * 5.8d-7
      Effko(ijk) =  0.3000000000000000D-30* (temp(ijk) / 3.d2)   &
                   **(- ( 0.3300000000000000D+01))
      Rapk(ijk) =  0.1500000000000000D-11* (temp(ijk) / 3.d2)   &
                    **(- ( 0.0000000000000000D+00))
      rk(ijk, 51) = (Effko(ijk) * SumM(ijk) /    &
                    ( 1.0d0 + Effko(ijk) * SumM(ijk) / Rapk(ijk))) *   &
                    0.6000D+00** (1.0d0 / (1.0d0 +    &
                     (LOG10(Effko(ijk) * SumM(ijk) / Rapk(ijk)))**2))
      rk(ijk, 52) = 1.5d-13 * (1.0d0 + 2.439d-20 * SumM(ijk))
      rk(ijk, 53) =  0.6000000000000000D-10
      rk(ijk, 54) =  0.0000000000000000D+00
      rk(ijk, 55) =  DEXP(-0.3943966082504782D+02   &
        + ( 0.2000000000000000D+01 * LOG(temp(ijk)))   &
        -  0.1361000000000000D+04/temp(ijk))
      rk(ijk, 56) =  DEXP(-0.3873183693007194D+02   &
        + ( 0.2000000000000000D+01 * LOG(temp(ijk)))   &
        -  0.4920000000000000D+03/temp(ijk))
      rk(ijk, 57) =   0.3760000000000000D-11   &
         * DEXP(-(  0.2600000000000000D+03 )/temp(ijk)) + & 
                 0.1700000000000000D-11   &
         * DEXP(-(  0.1550000000000000D+03 )/temp(ijk)) + & 
                 0.1210000000000000D-11   &
         * DEXP(-(  0.1250000000000000D+03 )/temp(ijk))
      rk(ijk, 58) =   0.1780000000000000D-11   &
         * DEXP(-( -0.4380000000000000D+03 )/temp(ijk)) + & 
                 0.6070000000000000D-12   &
         * DEXP(-( -0.5000000000000000D+03 )/temp(ijk)) + & 
                 0.0000000000000000D+00   &
         * DEXP(-( -0.4480000000000000D+03 )/temp(ijk))
      rk(ijk, 59) =  0.2540000000000000D-10   &
         * DEXP(-( -0.4100000000000000D+03 )/temp(ijk)) + & 
                 0.0000000000000000D+00   &
         * DEXP(-( -0.4400000000000000D+03 )/temp(ijk))
      rk(ijk, 60) =  0.3310000000000000D-11   &
        * DEXP(-( -0.3550000000000000D+03 )/temp(ijk)) + & 
                 0.3450000000000000D-12
      rk(ijk, 61) =  0.9999999999999999D-11
      rk(ijk, 62) =  DEXP(-0.2591722318817020D+02   &
         - ( -0.3310000000000000D+03 )/temp(ijk))
      rk(ijk, 63) =  DEXP(-0.3970958044115977D+02   &
        + ( 0.2000000000000000D+01 * LOG(temp(ijk)))   &
        +  0.9200000000000000D+02/temp(ijk))
      rk(ijk, 64) =  0.1860000000000000D-11   &
        * DEXP(-( -0.1750000000000000D+03 )/temp(ijk)) + & 
                 0.1320000000000000D-10
      rk(ijk, 65) =  DEXP(-0.2655601869289957D+02   &
         - ( -0.1900000000000000D+03 )/temp(ijk))
      rk(ijk, 66) =  DEXP(-0.2641908014195344D+02   &
         - ( -0.1900000000000000D+03 )/temp(ijk))
      rk(ijk, 67) =  0.1590000000000000D-13   &
        * DEXP(-( -0.5000000000000000D+03 )/temp(ijk)) + & 
                 0.3800000000000000D-13
      rk(ijk, 68) =  DEXP(-0.2596142928067470D+02   &
         - (  0.2600000000000000D+03 )/temp(ijk))
      rk(ijk, 69) =  DEXP(-0.2870983077730048D+02   &
         - (  0.1900000000000000D+04 )/temp(ijk))
      rk(ijk, 70) =  DEXP(-0.2729454887930734D+02   &
         - (  0.1900000000000000D+04 )/temp(ijk))
      rk(ijk, 71) =   0.1620000000000000D-11   &
         * DEXP(-(  0.1900000000000000D+04 )/temp(ijk)) + & 
                 0.0000000000000000D+00   &
         * DEXP(-(  0.1500000000000000D+03 )/temp(ijk)) + & 
                 0.1940000000000000D-13   &
         * DEXP(-(  0.1000000000000000D+04 )/temp(ijk))
      rk(ijk, 72) =  0.4920000000000000D-15
      rk(ijk, 73) =  0.4350000000000000D-17   &
         * DEXP( 0.2000000000000000D+01 * LOG(temp(ijk))   &
        -  0.2282000000000000D+04/temp(ijk)) + & 
                 0.1910000000000000D-13   &
         * DEXP(-(  0.4500000000000000D+03 )/temp(ijk)) + & 
                 0.1080000000000000D-14   &
         * DEXP(-( -0.4500000000000000D+03 )/temp(ijk)) + & 
                 0.0000000000000000D+00
      rk(ijk, 74) =  0.4000000000000000D-11   &
         * DEXP(-(  0.4460000000000000D+03 )/temp(ijk)) + & 
                 0.0000000000000000D+00   &
         * DEXP(-( -0.4900000000000000D+03 )/temp(ijk))
      rk(ijk, 75) =  DEXP(-0.3551694253050293D+02   &
         - (  0.5000000000000000D+03 )/temp(ijk))
      rk(ijk, 76) =  0.8170000000000000D-14   &
         * DEXP(-(  0.2580000000000000D+04 )/temp(ijk)) + & 
                 0.4320000000000000D-15   &
         * DEXP(-(  0.1800000000000000D+04 )/temp(ijk)) + & 
                 0.2870000000000000D-16   &
         * DEXP(-(  0.8450000000000000D+03 )/temp(ijk)) + & 
                 0.0000000000000000D+00   &
         * DEXP(-(  0.2283000000000000D+04 )/temp(ijk))
      rk(ijk, 77) =  0.7860000000000001D-14   &
         * DEXP(-(  0.1913000000000000D+04 )/temp(ijk)) + & 
                 0.0000000000000000D+00   &
         * DEXP(-(  0.7320000000000000D+03 )/temp(ijk))
      rk(ijk, 78) =  0.0000000000000000D+00   &
        * DEXP(-(  0.2112000000000000D+04 )/temp(ijk)) + & 
                 0.1380000000000000D-18
      rk(ijk, 79) =  DEXP(-0.3716986555487676D+02   &
         - (  0.1700000000000000D+04 )/temp(ijk))
      rk(ijk, 80) =  0.2000000000000000D-10
      rk(ijk, 81) =  0.9999999999999999D-11
      rk(ijk, 82) =  0.3600000000000000D-10
      rk(ijk, 83) =  DEXP(-0.3863712897853033D+02   &
         - ( -0.1044000000000000D+04 )/temp(ijk))
      rk(ijk, 83) = rk(ijk, 83) * SumM(ijk) * 0.2d0
      rk(ijk, 84) =  0.2800000000000000D-10
      Effko(ijk) =  0.9700000000000000D-28* (temp(ijk) / 3.d2)   &
                   **(- ( 0.5600000000000000D+01))
      Rapk(ijk) =  0.9300000000000000D-11* (temp(ijk) / 3.d2)   &
                    **(- ( 0.1500000000000000D+01))
      rk(ijk, 85) = (Effko(ijk) * SumM(ijk) /    &
                    ( 1.0d0 + Effko(ijk) * SumM(ijk) / Rapk(ijk))) *   &
                    0.5860D+00** (1.0d0 / (1.0d0 +    &
                     (LOG10(Effko(ijk) * SumM(ijk) / Rapk(ijk)))**2))
      rk(ijk, 86) =  DEXP( 0.6462080260895155D+02   &
         - (  0.1395400000000000D+05 )/temp(ijk))
      Effko(ijk) =  0.9700000000000000D-28* (temp(ijk) / 3.d2)   &
                   **(- ( 0.5600000000000000D+01))
      Rapk(ijk) =  0.9300000000000000D-11* (temp(ijk) / 3.d2)   &
                    **(- ( 0.1500000000000000D+01))
      facteur(ijk) = (Effko(ijk) * SumM(ijk) / ( 1.0d0 + Effko(ijk) * SumM(ijk) /    &
                 Rapk(ijk))) * 0.6d0 ** (1.0d0 / (1.0d0 +    &
                  (LOG10(Effko(ijk) * SumM(ijk) / Rapk(ijk)))**2))
      rk(ijk, 86) = facteur(ijk) * rk(ijk, 86)
      rk(ijk, 87) =  DEXP(-0.2619593659063922D+02   &
         - ( -0.1800000000000000D+03 )/temp(ijk))
      rk(ijk, 88) =  0.4360000000000000D-11
      rk(ijk, 89) =  0.6930000000000000D-11
      rk(ijk, 90) =  0.4000000000000000D-11
      rk(ijk, 91) =  0.4000000000000000D-11
      rk(ijk, 92) =  0.1220000000000000D-10
      rk(ijk, 93) =  0.4000000000000000D-11
      rk(ijk, 94) =  DEXP(-0.2859860514219026D+02   &
         - ( -0.8000000000000000D+03 )/temp(ijk))
      rk(ijk, 95) =  0.6160000000000000D-13   &
         * DEXP(-( -0.7000000000000000D+03 )/temp(ijk)) + & 
                 0.1520000000000000D-12   &
         * DEXP(-( -0.1300000000000000D+04 )/temp(ijk))
      rk(ijk, 96) =  DEXP(-0.2934027936364486D+02   &
         - ( -0.1300000000000000D+04 )/temp(ijk))
      rk(ijk, 98) =  DEXP(-0.2861185036894027D+02   &
         - ( -0.9800000000000000D+03 )/temp(ijk))
      rk(ijk, 99) =   0.5940000000000000D-12   &
         * DEXP(-( -0.5500000000000000D+03 )/temp(ijk)) + & 
                 0.1990000000000000D-15   &
         * DEXP(-( -0.2640000000000000D+04 )/temp(ijk)) + & 
                 0.5560000000000000D-13   &
         * DEXP(-( -0.1300000000000000D+04 )/temp(ijk))
      rk(ijk,100) =  DEXP(-0.2942678860655414D+02   &
         - ( -0.1300000000000000D+04 )/temp(ijk))
      rk(ijk,101) =  DEXP(-0.3002791688839384D+02   &
         - ( -0.4160000000000000D+03 )/temp(ijk))
      rk(ijk,102) =  0.1030000000000000D-13   &
         * DEXP(-( -0.1580000000000000D+03 )/temp(ijk)) + & 
                 0.6240000000000000D-13   &
         * DEXP(-( -0.4310000000000000D+03 )/temp(ijk)) + & 
                 0.1530000000000000D-13   &
         * DEXP(-( -0.4670000000000000D+03 )/temp(ijk)) + & 
                 0.4340000000000000D-14   &
         * DEXP(-( -0.6330000000000000D+03 )/temp(ijk))
      rk(ijk,103) =  DEXP(-0.2948253058956238D+02   &
         - ( -0.7080000000000000D+03 )/temp(ijk))
      rk(ijk,104) =  DEXP(-0.2962612150917463D+02   &
         - ( -0.7080000000000000D+03 )/temp(ijk))
      rk(ijk,105) =  DEXP(-0.3096643075705270D+02   &
         - ( -0.7080000000000000D+03 )/temp(ijk))
      rk(ijk,106) =   0.1770000000000000D-10   &
         * DEXP(-(  0.4400000000000000D+03 )/temp(ijk)) + & 
                 0.1480000000000000D-15   &
         * DEXP(-( -0.2510000000000000D+04 )/temp(ijk)) + & 
                 0.3100000000000000D-12   &
         * DEXP(-( -0.5080000000000000D+03 )/temp(ijk))
      rk(ijk,107) =  DEXP(-0.2982027752361559D+02   &
         - ( -0.7080000000000000D+03 )/temp(ijk))
      rk(ijk,108) =  0.4440000000000000D-13   &
         * DEXP(-( -0.2110000000000000D+03 )/temp(ijk)) + & 
                 0.2230000000000000D-12   &
         * DEXP(-( -0.4600000000000000D+03 )/temp(ijk)) + & 
                 0.4100000000000000D-13   &
         * DEXP(-( -0.5220000000000000D+03 )/temp(ijk)) + & 
                 0.1170000000000000D-13   &
         * DEXP(-( -0.6830000000000000D+03 )/temp(ijk))
      rk(ijk,109) =  DEXP(-0.2846113415156165D+02   &
         - ( -0.7650000000000000D+03 )/temp(ijk))
      rk(ijk,110) =  DEXP(-0.2790545796163031D+02   &
         - ( -0.7650000000000000D+03 )/temp(ijk))
      rk(ijk,111) =  DEXP(-0.2864437356064584D+02   &
         - ( -0.7650000000000000D+03 )/temp(ijk))
      rk(ijk,112) =  0.7730000000000000D-12   &
         * DEXP(-( -0.5300000000000000D+03 )/temp(ijk)) + & 
                 0.1700000000000000D-12   &
         * DEXP(-( -0.5650000000000000D+03 )/temp(ijk))
      rk(ijk,113) =  DEXP(-0.2835462750397320D+02   &
         - ( -0.7650000000000000D+03 )/temp(ijk))
      rk(ijk,114) =  DEXP(-0.3310607566097664D+02   &
         - ( -0.1000000000000000D+04 )/temp(ijk))
      rk(ijk,115) =  DEXP(-0.3132793274173975D+02   &
         - ( -0.1000000000000000D+04 )/temp(ijk))
      rk(ijk,116) =  0.1200000000000000D-11
      rk(ijk,117) =  0.1200000000000000D-11
      rk(ijk,118) =  0.1200000000000000D-11
      rk(ijk,119) =  0.1200000000000000D-11
      rk(ijk,120) =  0.1200000000000000D-11
      rk(ijk,121) =  0.3480000000000000D-11
      rk(ijk,122) =  0.1200000000000000D-11
      rk(ijk,123) =  DEXP(-0.2942678860655414D+02   &
         - ( -0.1300000000000000D+04 )/temp(ijk))
      rk(ijk,124) =  DEXP(-0.3274868498278332D+02   &
         - ( -0.1510000000000000D+04 )/temp(ijk))
      rk(ijk,125) =  DEXP(-0.3171146277298166D+02   &
         - ( -0.1560000000000000D+04 )/temp(ijk))
      rk(ijk,126) =  DEXP(-0.3717963534647257D+02   &
         - ( -0.2950000000000000D+04 )/temp(ijk))
      rk(ijk,127) =  0.4000000000000000D-11
      rk(ijk,128) =  0.1200000000000000D-11
       END DO
 
   END SUBROUTINE kinetic
 
  END MODULE mod_chem_spack_kinetic
 
